#include <Wire.h>
#include "Adafruit_TCS34725.h"
#include <Adafruit_NeoPixel.h>
 
#define florapin 8

// Pick analog outputs, for the UNO these three work well
// use ~560  ohm resistor between Red & Blue, ~1K for green (its brighter)
#define redpin 6
#define greenpin 10
#define bluepin 9
// for a common anode LED, connect the common pin to +5V
// for common cathode, connect the common to ground

// set to false if using a common cathode LED
#define commonAnode true 

#define colorAmount 2
#define colorDepth 50
#define colorDelay 2 // in secs

#define debug false

// our RGB -> eye-recognized gamma color
byte gammatable[256];
byte colorSets[colorAmount][3];
float r, g, b;

Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_50MS, TCS34725_GAIN_4X);
Adafruit_NeoPixel strip = Adafruit_NeoPixel(1, florapin, NEO_GRB + NEO_KHZ800);

void setup() {
  uint16_t clear, red, green, blue;
  uint32_t sum;
  float maxRgb;
  
  Serial.begin(9600);
  Serial.println("Color View Test!");

  if (tcs.begin()) {
    Serial.println("Found sensor");
  } else {
    Serial.println("No TCS34725 found ... check your connections");
    while (1); // halt!
  }

  if (colorDepth == 0) {
    Serial.println("Color depth should be greater than 0");
    while (1); // halt!
  }

  strip.begin();
  strip.show(); // Initialize all pixels to 'off'
  
  // use these three pins to drive an LED
  pinMode(redpin, OUTPUT);
  pinMode(greenpin, OUTPUT);
  pinMode(bluepin, OUTPUT);
  
  // thanks PhilB for this gamma table!
  // it helps convert RGB colors to what humans see
  for (int i=0; i<256; i++) {
    float x = i;
    x /= 255;
    x = pow(x, 2.5);
    x *= 255;
      
    if (commonAnode) {
      gammatable[i] = 255 - x;
    } else {
      gammatable[i] = x;      
    }
  }

  for (byte cpointer=0; cpointer<colorAmount; cpointer++) {

    tcs.setInterrupt(false);      // turn on LED
    delay(3000);  // takes 3sec to read 
    tcs.getRawData(&red, &green, &blue, &clear);
    tcs.setInterrupt(true);  // turn off LED

    if (debug) {
      Serial.print("C:\t"); Serial.print(clear);
      Serial.print("\tR:\t"); Serial.print(red);
      Serial.print("\tG:\t"); Serial.print(green);
      Serial.print("\tB:\t"); Serial.print(blue);
      Serial.println();
    }

    sum = clear;
    r = red; r /= sum;
    g = green; g /= sum;
    b = blue; b /= sum;

    if (debug) {
      Serial.print("\t\tR:\t"); Serial.print(r);
      Serial.print("\tG:\t"); Serial.print(g);
      Serial.print("\tB:\t"); Serial.print(b);
      Serial.println();
    }
    
    maxRgb = max(r, g);
    maxRgb = max(maxRgb, b);
    maxRgb = 255 / maxRgb;
    
    r *= maxRgb; g *= maxRgb; b *= maxRgb;
    colorSets[cpointer][0] = (byte) r;
    colorSets[cpointer][1] = (byte) g;
    colorSets[cpointer][2] = (byte) b;

    if (debug) {
      Serial.print("P:\t"); Serial.print(cpointer);
      Serial.print("\tR:\t"); Serial.print(colorSets[cpointer][0]);
      Serial.print("\tG:\t"); Serial.print(colorSets[cpointer][1]);
      Serial.print("\tB:\t"); Serial.print(colorSets[cpointer][2]);
      Serial.println();
    }

    delay(2000); // delay to change color
  }
}


void loop() {

  int dr=0, dg=0, db=0;
  byte tmpr, tmpg, tmpb;

  for (byte cpointer=0; cpointer<colorAmount; cpointer++) {

      dr = colorSets[cpointer][0] - (byte) r;
      dg = colorSets[cpointer][1] - (byte) g;
      db = colorSets[cpointer][2] - (byte) b;
      
      for (int j=1; j<=colorDepth; j++) {

        tmpr = (byte)(r + (dr * j/colorDepth));
        tmpg = (byte)(g + (dg * j/colorDepth));
        tmpb = (byte)(b + (db * j/colorDepth));
        
        if (debug) {
          Serial.println();
          Serial.print("\t\tR:\t"); Serial.print(tmpr);
          Serial.print("\tG:\t"); Serial.print(tmpg);
          Serial.print("\tB:\t"); Serial.print(tmpb);
        }

        colorWipe(strip.Color(
          (byte)(tmpr/10),
          (byte)(tmpg/10),
          (byte)(tmpb/10)
        ));
    
        analogWrite(redpin, gammatable[(byte)(tmpr)]);
        analogWrite(greenpin, gammatable[(byte)(tmpg)]);
        analogWrite(bluepin, gammatable[(byte)(tmpb)]);
        
        delay((int) (1000 * colorDelay / colorDepth));
      }

      r = colorSets[cpointer][0];
      g = colorSets[cpointer][1];
      b = colorSets[cpointer][2];
  }

}

void colorWipe(uint32_t c) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, c);
      strip.show();
  }
}

